<?php if(!defined('cfih') or !cfih) die('This file cannot be directly accessed.');
/**************************************************************************************************************
 *
 *   CF Image Hosting Script
 *   ---------------------------------
 *
 *   Author:    codefuture.co.uk
 *   Version:   1.6
 *
 *   You can download the latest version from: http://codefuture.co.uk/projects/imagehost/
 *
 *   Copyright (c) 2010-2012 CodeFuture.co.uk
 *   This file is part of the CF Image Hosting Script.
 *
 *   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 *   EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
 *   COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 *   WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF
 *   OR  IN  CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 *
 *   You may not modify and/or remove any copyright notices or labels on the software on each
 *   page (unless full license is purchase) and in the header of each script source file.
 *
 *   You should have received a full copy of the LICENSE AGREEMENT along with
 *   Codefuture Image Hosting Script. If not, see http://codefuture.co.uk/projects/imagehost/license/.
 *
 *
 *   ABOUT THIS PAGE -----
 *   Used For:     script functions
 *   Last edited:  21/06/2012
 *
 *************************************************************************************************************/

 
function error_header($code){
// Page errors
	$errorCode = array(
				'500' => array('HTTP/1.1 500 Internal Server Error', _T("error_500")),
				'404' => array('HTTP/1.1 404 Not Found', _T("error_404")),
				'403' => array('HTTP/1.1 403 Forbidden', _T("error_403")),
				'401' => array('HTTP/1.1 401 Unauthorized', _T("error_401")),
				'400' => array('HTTP/1.1 400 Bad Request',  _T("error_400"))
				);
	if (isset($errorCode[$code])) {
		header($errorCode[$code][0]);
		return $errorCode[$code][1];
	} else {
		return '';
	}
}


/***
 * themeSettings(theme name)
 * get any theme setting from themes/{theme name}/settings.php
 */
function themeSettings($themename){
	// check for theme settings 
	if (@include(CFROOTPATH.'themes/'.$themename.'/settings.php')){
		$return_theme = array();
		// check for theme settings
		if(isset($theme)){
			// theme settings index
			$themeOps = array(
									'designby','url','linktitle', // footer links
									'ads_header_bg','ads_header_link_color','ads_header_text_color','ads_header_url_color',
									'ads_index_bg','ads_index_link_color','ads_index_text_color','ads_index_url_color',
									'ads_thumb_bg','ads_thumb_link_color','ads_thumb_text_color','ads_thumb_url_color',
									'ads_gallery_bg','ads_gallery_link_color','ads_gallery_text_color','ads_gallery_url_color',
									'pagination_link_no','list_languages_over','captcha_light','widgit_row','gallery_row_no',
									'gallery_rows', 'thumb_option', 'thumb_max_width', 'thumb_max_height'
									);
			foreach($themeOps as $k ) {
				if(isset($theme[$k])) {
					$return_theme[$k] = $theme[$k];
				}else{
					$return_theme[$k] = '';
				}
			}
		}
		return $return_theme;
	}
	return array();
}


/***
 * theme_info(theme name)
 * about the theme (used on the admin Dashboard)
 */
function theme_info($themename){
	// load for theme settings (?)
	if (@include('themes/'.$themename.'/settings.php')){
		$return_theme = array();
		// check for theme info
		if(isset($theme_info)){
			// theme info index
			$themeOps = array('name','author','version','date','url','for');
			foreach($themeOps as $k ) {
				if(isset($theme_info[$k])) {
					$return_theme[$k] = $theme_info[$k];
				}else{
					$return_theme[$k] = '';
				}
			}
		}
		return $return_theme;
	}
	return array();
}


/***
 * listLanguages()
 * lists all language file found in the languages folder, used in header.php
 */
function listLanguages(){
	global $settings;
	$dir_list = opendir(CFROOTPATH.'languages/');
	$lang ='';
	while(false != ($file = readdir($dir_list))){
		if(($file != ".") && ($file != "..")){
			$lang_name = explode(".", $file);
			if (count($lang_name) > 2 && $lang_name[1].'.'.$lang_name[2] == 'lang.php'){
				 if($settings['SET_LANGUAGE']!=$lang_name[0])
					$lang .= '<a href="'.$settings['SET_SITEURL'].'/index.php?lang='.$lang_name[0].'" title="'.$lang_name[0].'" rel="nofollow"><img src="'.$settings['SET_SITEURL'].'/languages/'.$lang_name[0].'.png" alt="'.$lang_name[0].'" width="23" height="15" /></a> ';
			}
		}
	}
	if(empty($lang)) return false;
	return $lang;
}


/***
 * loadLanguage()
 * Load the language file what can be set by the admin in
 * settings of by the user from the language bar on the site
 */
function loadLanguage(){
	global $settings,$lang;

	// set default (admin set)
	$language = $settings['SET_LANGUAGE'];

	// check for user pick
	if(isset($_GET['lang']) && file_exists(CFROOTPATH.'languages/'.$_GET['lang'].'.lang.php')){
		$getLang = cl(removeSymbols(end(explode('/',$_GET['lang']))));
		setcookie('lang', $getLang, null);
		$language = $getLang;
	}

	// see if cookie has been set before
	if(isset($_COOKIE['lang']) && file_exists(CFROOTPATH.'languages/'.$_COOKIE['lang'].'.lang.php')){
		$language = $_COOKIE['lang'];
	}

	// check for lang files
	if(!@require_once(CFROOTPATH.'languages/'.$language.'.lang.php')){
		// fallback
		if(!@require_once(CFROOTPATH.'languages/english.lang.php')){
			die('Can\'t find english.lang.php');
		}
	}
}


/**
 * language function
 */
function _T($text){
	global $lang;
	if(empty($lang)) loadLanguage();
	if(isset($lang[$text])){
		return $lang[$text];
	}
	return $text;
}


/**
 * ImageWidget
 *	$numImg=[number of image to list], $return = [null(echo) | any thing other then null(return)]
 */
function ImageWidget($numImg=null, $return = null ){
	global $settings;
	if(!isset($settings['SET_IMAGE_WIDGIT']) || !$settings['SET_IMAGE_WIDGIT']) return;
	if($imageList = imageList('rand',$numImg)){

		$rand_widget = '<div id="randWidget" class="boxpanel">
			<h2 class="boxtitle">'._T("home_image_widgit").'</h2>
				<ul class="gallery">';
				
		foreach($imageList as $image){
		// get image address
			$thumb_url = imageAddress(3,$image,"dt");
		// get thumb page address
			$thumb_mid_link = imageAddress(2,$image,"pm");
		//see if there is a alt(title) if not use the image name
			$alt_text = ($image['alt'] !="" ? $image['alt']:$image['name']);
		//image list for page
			$rand_widget .= '
						<li><a href="'.$thumb_mid_link.'" title="'.$alt_text.'" class="thumb" >
							<img src="'.$thumb_url.'" alt="'.$alt_text.'" />
							</a><h2><a href="'.$thumb_mid_link.'" title="'.$alt_text.'">'.$alt_text.'</a></h2>
						</li>';
		}//	endfor
		$rand_widget .= '</ul><div class="clear"></div></div>';
		if(!is_null($return)) return $rand_widget;
		echo $rand_widget;
	}
}

function savefile($menu_array=array(),$fileaddress){
	if($fp = @fopen($fileaddress, 'w+')){
		fwrite($fp, serialize($menu_array));
		fclose($fp);
		return true;
	}else
		return false;
}

function report_img($id){
	global	$settings,$Err,$Suc;
	$id = cl($id);
	if(db_addReport($id)){
		$Suc['image_report'] = _T("site_gallery_report_suc");
		if ($settings['SET_EMAIL_REPORT'] && $settings['SET_CONTACT'] !='') {
			$subject = "Image Reported on ".$settings['SET_TITLE'];
			$message  = "reported image id: ".$id." \r\n";
			$message .= "reported on : ".$settings['SET_TITLE']." \r\n";
			$message .= "Admin Panel : ".$settings['SET_SITEURL']."/admin.php \r\n";
			$headers = "From:".$settings['SET_CONTACT']." <".$settings['SET_CONTACT'].">";
			mail($settings['SET_CONTACT'],$subject,$message,$headers);
		}
		return true;
	}
	$Err['cant_find_image'] = _T("site_gallery_report_err_find");
}

function shorturl_url( $url, $api=null){
	global $settings;

	$shorturl = '';
	if($api==null){
		if($settings['SET_SHORT_URL_API'] == 'b54'){
			$api = 'yourls';
			$settings['SET_SHORT_URL_API_URL'] = 'http://www.b54.in/api/';
		}else
			$api = $settings['SET_SHORT_URL_API'];
	}

	switch( $api ) {

		case 'yourls':
			$api_url = sprintf( $settings['SET_SHORT_URL_API_URL'] . '?username=%s&password=%s&url=%s&format=text&action=shorturl&source=plugin',$settings['SET_SHORT_URL_USER'], $settings['SET_SHORT_URL_PASS'], urlencode($url) );
			$shorturl = @file_get_contents( $api_url );
			break;

		case 'bitly':
			$api_url = sprintf( 'http://api.bit.ly/v3/shorten?longUrl=%s&login=%s&apiKey=%s&format=txt', urlencode($url), $settings['SET_SHORT_URL_USER'], $settings['SET_SHORT_URL_PASS'] );
			$shorturl = @file_get_contents( $api_url );
			break;

		case 'tinyurl':
			$api_url = sprintf( 'http://tinyurl.com/api-create.php?url=%s', urlencode($url) );
			$shorturl = @file_get_contents( $api_url );
			break;

		case 'isgd':
			$api_url = sprintf( 'http://is.gd/api.php?longurl=%s', urlencode($url) );
			$shorturl = @file_get_contents( $api_url );
			break;
		case 'googl':
			include './lib/goo.class.php';
			$googer = new GoogleURLAPI($settings['SET_SHORT_URL_PASS']);
			$shorturl = $googer->shorten($url);
			break;
		default:
			$shorturl='';
	}
	return $shorturl;
}

function error_note($myproblem=null,$ret = null) {
	global	$Err;
	return inote($Err,1,$ret);
}

function success_note($mysuccess=null,$ret = null) {
	global	$Suc;
	return inote($Suc,2,$ret);
}

/***
 * inote( note, type, return)
 * types
 * errors = [1 | err]
 * success = [2 | suc]
 * information = [any thing other then errors & success]
 */
function inote($mynotes,$type='info',$return = false) {
	if(empty($mynotes)) return;
	if(!is_array($mynotes)) $notes[] = $mynotes;
	else $notes = $mynotes;
	switch($type){
		case 1:
			$type = 'err';
			break;
		case 2:
			$type = 'suc';
			break;
		default:
			$type = 'info';
	}
	foreach ($notes as $k=>$note){
		$notes_html = '<div id="'.$k.'_'.$type.'" class="notification '.($type=='err'?'error':($type=='suc'?'success':'information')).'"><a class="close" href="#" alt="close" title="Close this notification"> </a>'.$note.'</div>';
	}
	if($return) return $notes_html ;
	echo $notes_html ;
}


/**
 * Clean Utility
 */
function cl($data){
	$data = stripslashes(strip_tags(html_entity_decode($data, ENT_QUOTES, 'UTF-8')));
	//$data = preg_replace('/[[:cntrl:]]/', '', $data); //remove control characters that cause interface to choke
	return $data;
}
/**
 * Encode Quotes
 */
function encode_quotes($text)  { 
	$text = strip_tags($text);
	if (version_compare(PHP_VERSION, "5.2.3")  >= 0) {	
		$text = htmlspecialchars($text, ENT_QUOTES, 'UTF-8', false);
	} else {	
		$text = htmlspecialchars($text, ENT_QUOTES, 'UTF-8');
	}
	return trim($text); 
} 

function removeSymbols($string) {
	$symbols = array('/','\\','\'','"',',','.','<','>','?',';',':','[',']','{','}','|','=','+','-','_',')','(','*','&','^','%','$','#','@','!','~','`');
	for ($i = 0; $i < count($symbols); $i++) {
		$string = str_replace($symbols[$i],' ',$string);
	}
	return trim($string);
}


function bookmarking($document_url,$document_title){
	global $settings;
	$ypid = (isset($settings['SET_ADDTHIS']) && !empty($settings['SET_ADDTHIS'])?'#pubid='.$settings['SET_ADDTHIS']:'');
	$text = '<div class="addthis">
	<!-- AddThis Button BEGIN -->
	<div class="addthis_toolbox addthis_default_style " addthis:url="'.$document_url.'" addthis:title="'.$document_title.'">
		<a class="addthis_button_preferred_1"></a>
		<a class="addthis_button_preferred_2"></a>
		<a class="addthis_button_email"></a>
		<a class="addthis_button_preferred_4"></a>
		<a class="addthis_button_preferred_5"></a>
		<a class="addthis_button_preferred_6"></a>
		<a class="addthis_button_preferred_7"></a>
		<a class="addthis_button_preferred_8"></a>
		<a class="addthis_button_compact"></a>
		<a class="addthis_counter addthis_bubble_style"></a>
	</div>
	<script type="text/javascript" src="http://s7.addthis.com/js/250/addthis_widget.js'.$ypid.'"></script>
	<!-- AddThis Button END -->
</div>';
	return $text;
}


function imageLinkCode($type,$imageaddress,$linkaddress=null,$alt=null){
	switch($type){
		case 'bbcode':
			return (!is_null($linkaddress)?'[URL='.$linkaddress.']':'').'[IMG]'.$imageaddress.'[/IMG]'.(!is_null($linkaddress)?'[/URL]':'');
			break;
		case 'html':
			return '&lt;a href=&quot;'.$linkaddress.'&quot; title=&quot;'.$alt.'&quot; &gt;&lt;img src=&quot;'.$imageaddress.'&quot; alt=&quot;'.$alt.'&quot; /&gt;&lt/a&gt;';
			break;
	}
}


function imageAddress($imgType,$image,$linktype=null){
	global $settings;

	$ext = 'html';
	$thumb_ext = isset($image['ext'])?strtolower($image['ext']):'';
	switch($imgType){
		case 1:
			$ext = $thumb_ext;
			if ($ext!='html')$fileaddress = CFIMAGEPATH.$image['id'].'.'.$ext;
			if (!isset($fileaddress) || !file_exists($fileaddress)) return false;
			break;
		case 2:
			$fileaddress = CFTHUMBPATH.$image['id'].'.';
			if($linktype=='dm')$ext = $thumb_ext;
			if(!file_exists($fileaddress.$thumb_ext)) $notfound =1;
			else $fileaddress .= $thumb_ext;
			break;
		case 3:
			$fileaddress = CFSMALLTHUMBPATH.$image['id'].'.';
			if($linktype=='dt')$ext = $thumb_ext;
			if(!file_exists($fileaddress.$thumb_ext)) $notfound =1;
			else $fileaddress .= $thumb_ext;
			break;
		case 4:
			$ext = $thumb_ext;
			$fileaddress = CFIMAGEPATH.$image['id'].'.'.$ext;
			if (!isset($fileaddress) || !file_exists($fileaddress)) return false;
			break;
	}

// look for the right file ext
	if(isset($notfound)){
		foreach (array('png','jpg','jpeg','gif') as $fileExt){
			if ($thumb_ext != $fileExt && file_exists($fileaddress.$fileExt)){
				$fileaddress .= $fileExt;
				if($linktype=='dt' || $linktype=='dm') $ext = $fileExt;
				break;
			}
		}
	}

	if (isset($fileaddress)){
		if (!is_null($linktype)){
			if($settings['SET_MOD_REWRITE']){
				$pieces = explode(".", $image['name']);
				return $settings['SET_SITEURL'].'/'.$linktype.'/'.$image['id'].'/'.str_replace(" ","_",$pieces[0]).'.'.$ext;
			}else{
				return $settings['SET_SITEURL'].'/'.($ext!='html'?'image.php':'').'?'.$linktype.'='.$image['id'];
			}
		}
		elseif (is_null($linktype)){
			return str_replace(CFROOTPATH, "", $fileaddress);
		}
	}
	return false;
}


function order_by(&$db,$field, $order = 123) {
	if ($order == 'ASC' || $order == 123)$order = '$a,$b';
	if ($order == 'DESC' || $order == 321)$order = '$b,$a';
	$code = "return strnatcmp(\$a['$field'], \$b['$field']);";
	@usort($db, create_function($order, $code));
}


/***
 * removeImage()
 * user image romove function
 */
function removeImage($imageDeleteCode=null){
	global $settings,$Suc,$Err;
	
	if (is_null($imageDeleteCode) && isset($_GET['d'])){
		$imageDeleteCode = $_GET['d'];
	}elseif (is_null($imageDeleteCode)){
		return;
	}
	
	if (preg_replace("/[^0-9A-Za-z]/","",$imageDeleteCode) != $imageDeleteCode || empty($imageDeleteCode)){
		$Err['delete_image'] = _T("site_index_delete_image_err_not_found");
		return false;
	}

	if (!$image = getImage($imageDeleteCode,'deleteid')){
		$Err['delete_image'] = _T("site_index_delete_image_err_not_found");
		$_GET['err'] = '404';// not found (404)page error
		return false;
	}

// Remove Image
	if(@unlink(imageAddress(1,$image))){
		$Suc['delete_image'] = _T("site_index_delete_image_suc");
	}
// Remove small thumb
	@unlink(imageAddress(3,$image));
// Remove thumb
	@unlink(imageAddress(2,$image));

// Remove link from array
	if (!removeImageDb($image['id'])){
		$Err['delete_image'] = _T("site_index_delete_image_err_db");
		return false;
	}

// Remove bw db
	@unlink(CFBANDWIDTHPATH.$image['id'].'_imgbw.db');
	return true;

}


/***
 * autoDelete()
 * Checks for image that have not been viewed in for a set period of time then removes/deletes them
 * The set period of time $settings['SET_AUTO_DELETED_JUMP'] is set in admin setttings...
 */
function autoDeleted(){
	global $settings,$Suc,$Err;

// check auto remove image is on
	if(!$settings['SET_AUTO_DELETED']) return;

// make file name
	$checkfor = CFDATAPATH.'ad'.date($settings['SET_AUTO_DELETED_JUMP']);

// check to see if it's been run for this time period
	if(is_file($checkfor)) return;

// get image index
	$db_img = imageList(0,'all');
	
// check image index for images
	if(empty($db_img)||count($db_img)<1) return;
	
	$delete_time = ($settings['SET_AUTO_DELETED_TIME'] * 86400) - time();


// check when image was last viewed
	foreach ($db_img as $k => $image){

	// check to see if the image has been uploaded for long then the period
		if( $delete_time > $image['added']){

		// image viewed db address
			$img_view_db = CFBANDWIDTHPATH.$image['id'].'_imgbw.db';

		// check to see if there is a images viewed db file
			if(is_file($img_view_db)){
			
			// read files last modified date (as this is most likely the last viewed date)
				if($last_modified = filemtime ( $img_view_db )){
				
				// check to see if it has not been view for the period
					if( $delete_time > $last_modified){
						$delete_id[$image['id']] = array('deleteid' => $image['deleteid']);// add image to remove array
					}

				}
			// if we can't read the date read the data from the db
				else{
					$db_count = db_imageCounterList(null,$image['id']);
					if (isset($db_count['date'])){
						if(  $delete_time > $db_count['date']){
							$delete_id[$image['id']] = array('deleteid' => $image['deleteid']);// add image to remove array
						}
					}

				}	
			}
		// if it's not been viewed
			else{
				$delete_id[$image['id']] = array('deleteid' => $image['deleteid']);// add image to remove array
			}
		}
	}

//remove images
	if(!empty($delete_id)){
		foreach ($delete_id as $k => $image){
			removeImage($image['deleteid']);
		}
	}

// remove image removed meg
	$Suc = array();

// make lasted checked file
	if(savefile(array(),CFDATAPATH.'ad'.date($settings['SET_AUTO_DELETED_JUMP']))){
	// check for old file
		if(is_file(CFDATAPATH.'ad'.(date($settings['SET_AUTO_DELETED_JUMP'])-1))){
		// remove old file if found
			@unlink (CFDATAPATH.'ad'.(date($settings['SET_AUTO_DELETED_JUMP'])-1));
		}
	}

}


/***
 * format_size ()
 * Convert size in bytes to a human readable format 
 */
function format_size($size="",$file="") {
	if (empty($size) && !empty($file)) $size = @filesize($file);

	if (strlen($size) <= 9 && strlen($size) >= 7){
		$img_size = substr(number_format($size / 1048576,2), -2) == '00' 
					? number_format($size / 1048576,0):number_format($size / 1048576,2);
		$img_size .= " MB";
	}elseif (strlen($size) >= 10){
		$img_size = substr(number_format($size / 1073741824,2), -2) == '00' 
					? number_format($size / 1073741824,0):number_format($size / 1073741824,2);
		$img_size .= " GB";
	}else $img_size = number_format($size / 1024,0)." kb";

	return $img_size;
}

function checklogin() {
	global $settings;
	if(isset($_SESSION['loggedin'])){
		if ($_SESSION['set_name'] == md5($settings['SET_USERNAME'].$settings['SET_SALTING'].$settings['SET_PASSWORD'])){
			return true;
		}else{
			session_unset();
			session_destroy();
		}
	}
	return false;
}

function pagination($pageOn,$itemsOnPage,$itemCount,$pageAddress,$numberOfPageLinks = null){
	$pageOn++;// add 1 to fix page number

// the number of links to show
	if(is_null($numberOfPageLinks))$numberOfPageLinks = 11;

// work out the No. of Pages
	$noOfPages = ceil($itemCount/$itemsOnPage);

// On page * of **
	$pagination = '<div class="pagination"><span class="pagecount">'.sprintf(_T("pagination_page_of"), $pageOn, $noOfPages).'</span>' ;

//first and prev buttons
	$pagination.= ($pageOn>1) ? '<a href="'.sprintf($pageAddress, 1).'" title="'._T("pagination_page_first_tip").'">'._T("pagination_page_first").'</a><a href="'.sprintf($pageAddress, ($pageOn-1)).'" title="'._T("pagination_previous_page_tip").'">-</a>':'';


	$numberToList = $noOfPages > ($numberOfPageLinks-1) ? ($numberOfPageLinks-1) :($noOfPages-1);
	$listStart = (($pageOn-(($numberOfPageLinks-1)/2)) < 1) ? 1 : (($pageOn+(($numberOfPageLinks-1)/2))>$noOfPages ? ($noOfPages-$numberToList):($pageOn-(($numberOfPageLinks-1)/2)));

	for ($i = $listStart; $i <= ($listStart+$numberToList); $i++) {
		$pagination .=($i==$pageOn ? '<span class="current">'.$i.'</span>':'<a href="'.sprintf($pageAddress, $i).'" title="'.sprintf(_T("pagination_page_tip"),$i).'">'.$i.'</a>');
	}

// next and last pages
	$pagination .= ($pageOn) < $noOfPages ? '<a href="'.sprintf($pageAddress, ($pageOn+1)).'" title="'._T("pagination_next_page_tip").'">+</a><a href="'.sprintf($pageAddress, $noOfPages).'" title="'._T("pagination_page_last_tip").'">'._T("pagination_page_last").'</a>':'';
	$pagination .='</div>';

	return $pagination;
}

function saveSettings($address,$settings){

$setFile ='<?php

// stop direct access
	if(!defined(\'cfih\') or !cfih) exit("Direct access not permitted.");

// settings
';

	foreach ($settings as $n => $v){
		$setFile .= '	$settings[\''.$n.'\'] = '.(is_numeric($v)?$v:'\''.encode_quotes($v).'\'').";\n";
	}

	if($fp = @fopen($address, 'w+')){
		fwrite($fp, $setFile);
		fclose($fp);
		return true;
	}else
		return false;

}

function flushNow($now = null){
	echo(str_repeat(' ',256));
	// check that buffer is actually set before flushing
	if (ob_get_length()){           
		@ob_flush();
		@flush();
		@ob_end_flush();
	}
	@ob_start();
	if(is_null($now)) usleep(rand(2,4)*100000);
}


/**
 * Add code line
 */
function add_code_line($hook_name, $added_line) {
	global $codelines;
	$bt = debug_backtrace();
	$shift=count($bt) - 4;	// plugin name should be  
  	$caller = array_shift($bt);
	$realPathName=pathinfo_filename($caller['file']);
	$realLineNumber=$caller['line'];
	while ($shift > 0) {
		 $caller = array_shift($bt);
		 $shift--;
	}
  	$pathName= pathinfo_filename($caller['file']);
	$codelines[] = array(
		'hook' => $hook_name,
		'codeline' => $added_line,
		'file' => $pathName.'.php',
		'line' => $caller['line']
	); 
}


/**
 * Add Action
 */
function add_action($hook_name, $added_function, $args = array()) {
	global $plugins;
  
	$bt = debug_backtrace();
	$shift=count($bt) - 4;	// plugin name should be  
  	$caller = array_shift($bt);
	$realPathName=pathinfo_filename($caller['file']);
	$realLineNumber=$caller['line'];
	while ($shift > 0) {
		 $caller = array_shift($bt);
		 $shift--;
	}
  	$pathName= pathinfo_filename($caller['file']);
	$plugins[] = array(
		'hook' => $hook_name,
		'function' => $added_function,
		'args' => (array) $args,
		'file' => $pathName.'.php',
		'line' => $caller['line']
	); 
}


/**
 * Execute Action
 */
function exec_action($a) {
	global $plugins,$codelines;

	if(!empty($plugins)){
		foreach ($plugins as $hook)	{
			if ($hook['hook'] == $a) {
				call_user_func_array($hook['function'], $hook['args']);
			}
		}
	}

	if(!empty($codelines)){
		foreach ($codelines as $hook)	{
			if ($hook['hook'] == $a) {
				echo $hook['codeline']."\r\n";
			}
		}
	}
}


/**
 * getAdCode
 */
function get_ad_code($name,$decode = true){
	global $ads,$settings,$themeSet;
		if(empty($ads)){
	// Adsense ads
		if(!empty($settings['SET_GOOGLE_ADS'])){// && !checklogin()){
			include_once('./AdSense.php'); // AdSense
		}
	// Your ads
		elseif(is_file(CFDATAPATH.'ads.php')){
			@require_once(CFDATAPATH.'ads.php');
		}
	}
	if(isset($ads[$name])){
		if($decode) return safe_strip_decode($ads[$name]);
		else return $ads[$name];
	}
	return false;
}


/**
 * Safe Pathinfo Filename
 */
function pathinfo_filename($file) {
	if (defined('PATHINFO_FILENAME')) return pathinfo($file,PATHINFO_FILENAME);
	$path_parts = pathinfo($file);

	if(isset($path_parts['extension']) && ($file!='..')){
	  return substr($path_parts['basename'],0 ,strlen($path_parts['basename'])-strlen($path_parts['extension'])-1);
	} else{
	  return $path_parts['basename'];
	}
}


/**
 * Strip Quotes
 */
function safe_slash_html($text) {
	if (get_magic_quotes_gpc()==0) {
		$text = addslashes(htmlspecialchars($text, ENT_QUOTES, 'UTF-8'));
	} else {
		$text = htmlspecialchars($text, ENT_QUOTES, 'UTF-8');
	}
	$text = str_replace(chr(12), '', $text);
	$text = str_replace(chr(3), ' ', $text);
	return $text;
}


/**
 * Safe StripSlashes HTML Decode
 */
function safe_strip_decode($text) {
	if (get_magic_quotes_gpc()==0) {
		$text = htmlspecialchars_decode($text, ENT_QUOTES);
	} else {
		$text = stripslashes(htmlspecialchars_decode($text, ENT_QUOTES));
	}
	return $text;
}


/**
 * Validate Email Address
 */
function check_email_address($email) {
	if (function_exists('filter_var')) {
		// PHP 5.2 or higher
		return (!filter_var((string)$email,FILTER_VALIDATE_EMAIL)) ? false: true;
	} else {
		// old way
		if (!preg_match("/[^@]{1,64}@[^@]{1,255}$/", $email)) {
			return false;
		}
		$email_array = explode("@", $email);
		$local_array = explode(".", $email_array[0]);
		for ($i = 0; $i < sizeof($local_array); $i++) {
			if (!preg_match("/(([A-Za-z0-9!#$%&'*+\/\=?^_`{|}~-][A-Za-z0-9!#$%&'*+\/\=?^_`{|}~\.-]{0,63})|(\"[^(\\|\")]{0,62}\"))$/", $local_array[$i])) {
				return false;
			}
		}
		if (!preg_match("/\[?[0-9\.]+\]?$/", $email_array[1])) {
			$domain_array = explode(".", $email_array[1]);
			if (sizeof($domain_array) < 2) {
				return false; // Not enough parts to domain
			}
			for ($i = 0; $i < sizeof($domain_array); $i++) {
				if (!preg_match("/(([A-Za-z0-9][A-Za-z0-9-]{0,61}[A-Za-z0-9])|([A-Za-z0-9]+))$/", $domain_array[$i])) {
					return false;
				}
			}
		}
		return true;
	  }
}

/**
 * Add Trailing Slash
 */
function tsl($path) {
	if( substr($path, strlen($path) - 1) != '/' ) {
		$path .= '/';
	}
	return $path;
}


if (!function_exists("preprint")) {
	function preprint($s, $return=false, $height='200') {
		$x = '<pre style="height:'.$height.'px; overflow: auto; border: 1px solid rgb(153, 153, 153); padding: 5px; width: 500px;">';
		$x .= print_r($s, 1);
		$x .= "</pre>";
		if ($return) return $x;
		else print $x;
	}
}